var app = angular.module('tc-explorer', ['ngRoute'])

app.config(($routeProvider) => {
    $routeProvider
        .when('/connect', {templateUrl: 'views/connect.html'})
        .when('/menu', {templateUrl: 'views/menu.html'})
        .when('/preferences', {templateUrl: 'views/preferences.html'})
        .when('/detail', {templateUrl: 'views/detail.html'})
        .when('/databases', {templateUrl: 'views/databases.html'})
        .when('/tables', {templateUrl: 'views/tables.html'})
        .when('/rows', {templateUrl: 'views/rows.html'})
        .when('/custom', {templateUrl: 'views/custom.html'})
        .otherwise({redirectTo: '/connect'})
})

app.controller('connectCtrl', ['$scope', '$rootScope', '$location', ($scope, $rootScope, $location) => {
    $scope.connect = async () => {
        $scope.connecting = true

        try {
            const response = await fetch('/api/permission')
            const data = await response.text()

            if (data === 'ok') {
                $scope.connecting = false
                $location.path('/menu')
                $scope.$apply()
            } else {
                $rootScope.errorMessage = 'Connection refused'
                $scope.connecting = false
                $scope.$apply()
            }
        } catch (ex) {
            console.log(ex)
            $rootScope.errorMessage = 'Connection timeout'
            $scope.connecting = false
            $scope.$apply()
        }
    }
}])

app.controller('menuCtrl', ['$scope', '$rootScope', '$location', ($scope, $rootScope, $location) => {
    $scope.init = () => {
        $rootScope.previousFileID = null
        $rootScope.previousDBID = null
        $rootScope.previousTableID = null
    }

    $scope.goToDatabase = () => {
        $location.path('/databases')
    }

    $scope.goToSettings = () => {
        $location.path('/preferences')
    }
}])

app.controller('preferencesCtrl', ['$scope', '$rootScope', '$location', ($scope, $rootScope, $location) => {
    $scope.init = async () => {
        try {
            const response = await fetch('/api/preferences')
            const data = await response.json()

            if (data) {
                $rootScope.files = data
                var i = 0
                $rootScope.files.forEach((file) => {
                file.id = i++
                file.lastModified = new Date(file.lastModified).toLocaleString('en-SE')
                })

                $scope.$apply()
            }
        } catch (ex) {
            console.log(ex)
            $rootScope.errorMessage = 'You are not connected to a phone'
            $location.path('/connect')
            $scope.$apply()
        }
    }

    $scope.goBack = () => {
        $location.path('/menu')
    }

    $scope.selectFile = (id) => {
        const file = $rootScope.files.find(f => f.id === id)
        if (file) {
            $rootScope.selectedFile = file
            $location.path('/detail')
        }
    }
}])

app.controller('detailCtrl', ['$scope', '$rootScope', '$location', ($scope, $rootScope, $location) => {
    $scope.init = () => {
        if (!$rootScope.files) {
            $location.path('/menu')
        }
    }

    $scope.typeOf = (obj) => {
        return typeof obj
    }

    $scope.displayAsDate = (obj) => {
        return new Date(obj).toLocaleString('en-SE')
    }

    $scope.isTimestamp = (key) => {
        const words = [
            'until',
            'last',
            'timestamp',
            'time'
            ]

        for (var i in words) {
            if (key.toLowerCase().includes(words[i])) {
                return true
            }
        }
    }

    $scope.goBack = () => {
        $rootScope.previousFileID = $scope.selectedFile.id
        $location.path('/preferences')
    }

    $scope.updateRow = async (key, value) => {
        let uri = $rootScope.selectedFile.packageName
            + '.'
            + $rootScope.selectedFile.fileName
            + '/'
            + key
            + '/'
            + typeof value
            + '/'
            + value

        try {
            const response = await fetch('/api/preferences/' + uri, {method: 'POST'})
            const data = await response.text()

            console.log(data)
        } catch (ex) {
            console.log('Error while updating fields : ' + ex)
        }
    }
}])

app.controller('databaseCtrl', ['$scope', '$rootScope', '$location', ($scope, $rootScope, $location) => {
    $scope.init = async () => {
        try {
            const response = await fetch('/api/databases')
            const data = await response.json()

            if (data) {
            $rootScope.previousTableID = null
            $rootScope.previousViewID = null
            $rootScope.files = data
            var i = 0
            $rootScope.files.forEach((file) => {
                file.id = i++
                file.fileName = file.mFileName
                file.lastModified = new Date(file.mLastModified).toLocaleString('en-SE')
            })

            $scope.$apply()
            }
        } catch (ex) {
            console.log(ex)
            $rootScope.errorMessage = 'You are not connected to a phone'
            $location.path('/connect')
            $scope.$apply()
        }
    }

    $scope.selectFile = (id) => {
        const file = $rootScope.files.find(f => f.id === id)
        if (file) {
            $rootScope.selectedDB = file
            $location.path('/tables')
        }
    }

    $scope.goBack = () => {
        $location.path('/menu')
    }
}])

app.controller('tablesCtrl', ['$scope', '$rootScope', '$location', ($scope, $rootScope, $location) => {
    $scope.init = async () => {
        try {
            const response = await fetch('/api/databases/' + $rootScope.selectedDB.fileName)
            const data = await response.json()

            if (data) {
                $scope.tables = data
                $scope.tables.shift()
                console.log(data)
                $scope.$apply()
            }
        } catch (ex) {
            console.log(ex)
            $rootScope.errorMessage = 'You are not connected to a phone'
            $location.path('/connect')
            $scope.$apply()
        }

        try {
            const response = await fetch('/api/databases/' + $rootScope.selectedDB.fileName + '/views')
            const data = await response.json()

            if (data) {
                $scope.views = data
                $scope.views.shift()
                console.log(data)
                $scope.$apply()
            }
        } catch (ex) {
            console.log(ex)
            $rootScope.errorMessage = 'You are not connected to a phone'
            $location.path('/connect')
            $scope.$apply()
        }
    }

    $scope.selectTable = (id) => {
        $rootScope.selectedTable = $scope.tables[id]
        $rootScope.previousTableID = id
        $location.path('/rows')
    }

    $scope.selectView = (id) => {
        $rootScope.selectedTable = $scope.views[id]
        $rootScope.previousViewID = id
        $location.path('/rows')
    }

    $scope.goBack = () => {
        $rootScope.previousDBID = $rootScope.selectedDB.id
        $location.path('/databases')
    }

    $scope.customQuery = () => {
        $location.path('/custom')
    }

    $scope.downloadDB = (name) => {
        try {
            window.open('/api/download/' + name)
        } catch (ex) {
            console.log(ex)
        }
    }
}])

app.controller('rowsCtrl', ['$scope', '$rootScope', '$location', ($scope, $rootScope, $location) => {
    $scope.init = async () => {
        $scope.page = 0
        $scope.showQuery = false
        $scope.getRows()
        $scope.getPagesCount()
    }

    $scope.getPagesCount = async () => {
        try {
            const response = await fetch('/api/databases/' + $rootScope.selectedDB.fileName + '/' + $rootScope.selectedTable + '/pages')
            const data = await response.json()

            if (data) {
                $scope.maxPages = data - 1
                $scope.$apply()
            }
        } catch (ex) {
            console.log(ex)
            $rootScope.errorMessage = 'You are not connected to a phone'
            $location.path('/connect')
            $scope.$apply()
        }
    }

    $scope.getRows = async () => {
        try {
            const response = await fetch('/api/databases/' + $rootScope.selectedDB.fileName + '/' + $rootScope.selectedTable + '/' + $scope.page)
            const data = await response.json()

            if (data) {
                $scope.files = data
                $scope.header = $scope.files.shift()
                console.log(data)
                window.scrollTo(0, 0)
                $scope.$apply()
            }
        } catch (ex) {
            console.log(ex)
            $rootScope.errorMessage = 'You are not connected to a phone'
            $location.path('/connect')
            $scope.$apply()
        }
    }

    $scope.goBack = () => {
        $location.path('/tables')
    }

    $scope.previousPage = async () => {
        if ($scope.page > 0) {
            $scope.page--
            $scope.getRows()
        }
    }

    $scope.nextPage = async () => {
        if ($scope.page < $scope.maxPages) {
            $scope.page++
            $scope.getRows()
        }
    }

    $scope.equalsRows = (row1, row2) => {
        if (!row1 || !row2) {
            return false
        } else {
            return row1[0] === row2[0] // Compare _ID
        }
    }

    $scope.editRow = (row) => {
        $scope.selectedRow = angular.copy(row)
        $scope.$apply()
    }

    $scope.saveRow = async () => {
        try {
            var query = 'UPDATE ' + $scope.selectedTable + ' SET '

            for (var i = 1; i < $scope.header.length; i++) {

                if ($scope.selectedRow[i]) {
                    query += ($scope.header[i] + '="' + $scope.selectedRow[i] + '", ')
                } else {
                    query += ($scope.header[i] + '="", ')
                }

                if (i === $scope.header.length - 1) {
                    query = query.slice(0, -2)
                    query += (' WHERE _id=' + $scope.selectedRow[0])

                    const response = await fetch('/api/databases/' + $rootScope.selectedDB.fileName + '/query/' + query)
                    const data = await response.json()

                    $scope.selectedRow = null
                    $scope.getRows()
                    $scope.$apply()
                }
            }
        } catch (ex) {
            console.log(ex)
        }
    }

    $scope.cancelRow = () => {
        $scope.selectedRow = null
        $scope.$apply()
    }
}])

app.controller('customCtrl', ['$scope', '$rootScope', '$location', ($scope, $rootScope, $location) => {
    $scope.init = () => {
        $scope.files = null
        $scope.showQuery = true
    }

    $scope.goBack = () => {
        $location.path('/tables')
    }

    $scope.runQuery = async () => {
        var text = null
        try {
            const response = await fetch('/api/databases/' + $rootScope.selectedDB.fileName + '/query/' + $scope.query)
            text = await response.text()
            const data = JSON.parse(text)

            if (data) {
                $scope.sqlError = null
                $scope.files = data
                $scope.header = $scope.files.shift()
                console.log(data)
                window.scrollTo(0, 0)
                $scope.$apply()
            }
        } catch (ex) {
            console.log(ex)
            $scope.sqlError = text
            $scope.files = null
            $scope.$apply()
        }
    }
}])